@@ -9,16 +9,11 @@
 
 """Invenio CLI Docker Compose class."""
 
-import io
-import logging
-import os
 import re
-import subprocess
-import tarfile
 
 import docker
 
-from .log import LogPipe
+from .process import ProcessResponse, run_cmd, run_interactive
 
 DOCKER_COMPOSE_VERSION_DASH = '1.21.0'
 
@@ -33,58 +28,14 @@ class DockerHelper(object):
         self.local = local
         self.docker_client = docker.from_env()
 
-        # Set as INFO to allow all logs to be sent
-        # TODO: revisit when dealing with logs
-        self.log_config = log_config
-        if log_config:
-            logging.basicConfig(filename=self.log_config.logfile,
-                                level=logging.INFO)
-
-    def start_containers(self):
-        """Start containers according to the specified environment."""
-        command = [
-            'docker-compose',
-            '--file',
-            'docker-compose.yml' if self.local else 'docker-compose.full.yml',
-            'up',
-            # NOTE: docker-compose is smart about not rebuilding an image if
-            #       there is no need to, so --build is not a slow default.
-            '--build',
-            '-d'  # --detach not supported in 1.17.0
-        ]
-        # On a re-run everything is good.
-        subprocess.run(command, check=True)
-
-    def stop_containers(self):
-        """Stop currently running containers."""
-        command = [
-            'docker-compose',
-            '--file',
-            'docker-compose.full.yml',
-            'stop'
-        ]
-
-        subprocess.call(command)
-
-    def destroy_containers(self):
-        """Stop and remove all containers, volumes and images."""
-        command = ['docker-compose', '--file', 'docker-compose.full.yml',
-                   'down', '--volumes']
-
-        subprocess.call(command)
-
     def _normalize_name(self, project_shortname):
         """Normalize the container name according to the compose version.
 
         Docker-Compose introduced support for dash and underscore in
         version 1.21.0.
         """
-        dc_version_command = subprocess.Popen(['docker-compose', '--version'],
-                                              stdout=subprocess.PIPE)
-        dc_version_string = dc_version_command.communicate()[0]
-        dc_version_string = dc_version_string.decode("utf-8").strip()
-
-        groups = re.search(r'1.[0-9]*.[0-9]*', dc_version_string)
+        dc_version_string = run_cmd(['docker-compose', '--version'])
+        groups = re.search(r'1.[0-9]*.[0-9]*', dc_version_string.output)
         dc_version = groups.group(0)
 
         if dc_version < DOCKER_COMPOSE_VERSION_DASH:
@@ -92,29 +43,68 @@ class DockerHelper(object):
         else:
             return project_shortname
 
-    def copy2(self, src_path, dst_path):
-        """Copy a file into the path of the specified container."""
-        container_name = '{}_web-ui_1'.format(self.container_prefix)
-        container_path = "{}:{}".format(container_name, dst_path)
+    def build_images(self, pull=False, cache=True):
+        """Build images.
 
-        commands = ['docker', 'cp', str(src_path), container_path]
-        subprocess.run(commands, check=True)
+        :param pull: Adds --pull to the docker-compose command.
+        :param cache: Removes --no-cache to the docker-compose command.
+        """
+        command = [
+            'docker-compose',
+            '--file',
+            'docker-compose.full.yml',
+            'build',
+        ]
+        if pull:
+            command.append('--pull')
+        if not cache:
+            command.append('--no-cache')
+
+        # FIXME: To get real-time output
+        return run_interactive(command)
+
+    def start_containers(self, app_only=False):
+        """Start containers according to the specified environment.
+
+        :param app_only: Boot up only ui and api containers.
+        """
+        command = [
+            'docker-compose',
+            '--file',
+            'docker-compose.yml' if self.local else 'docker-compose.full.yml',
+            'up',
+            '-d'  # --detach not supported in 1.17.0
+        ]
+
+        if app_only:
+            command.extend(['web-ui', 'web-api'])
+
+        return run_cmd(command)
+
+    def stop_containers(self):
+        """Stop currently running containers."""
+        command = ['docker-compose', '--file', 'docker-compose.full.yml',
+                   'stop']
+        return run_cmd(command)
+
+    def destroy_containers(self):
+        """Stop and remove all containers, volumes and images."""
+        command = ['docker-compose', '--file', 'docker-compose.full.yml',
+                   'down', '--volumes']
+
+        return run_cmd(command)
 
     def execute_cli_command(self, project_shortname, command):
         """Execute an invenio CLI command in the API container."""
         container_name = '{}_web-ui_1'.format(self.container_prefix)
         container = self.docker_client.containers.get(container_name)
-
         status = container.exec_run(
             cmd='/bin/bash -c "{}"'.format(command.replace('"', '\\"')),
             tty=True,
             stdout=True,
             stderr=True)
-
-        if self.log_config and self.log_config.verbose:
-            print(status.output.decode("utf-8"))
-        else:
-            level = logging.INFO if status.exit_code == 0 else logging.ERROR
-            logging.log(level, status.output.decode("utf-8").strip('\n'))
-
-        return status.exit_code
+        # FIXME: What happens when exec_run fails? handle exception.
+        return ProcessResponse(
+            output=status.output.decode("utf-8").strip(),
+            status_code=status.exit_code
+        )
